@@ -4,52 +4,52 @@ import time
 import collections
 
 # Dependencies
-from PyQt5 import QtCore
 import pyblish.logic
 
 # Local libraries
 from . import util, models, version, settings
+from .vendor.Qt5 import QtCore
 
-qtproperty = util.pyqtConstantProperty
+qtproperty = util.qtConstantProperty
 
 
 class Controller(QtCore.QObject):
     """Communicate with QML"""
 
-    # PyQt Signals
-    info = QtCore.pyqtSignal(str, arguments=["message"])
-    error = QtCore.pyqtSignal(str, arguments=["message"])
+    # Signals
+    info = QtCore.Signal(str, arguments=["message"])
+    error = QtCore.Signal(str, arguments=["message"])
 
-    show = QtCore.pyqtSignal()
-    hide = QtCore.pyqtSignal()
+    show = QtCore.Signal()
+    hide = QtCore.Signal()
 
-    firstRun = QtCore.pyqtSignal()
+    firstRun = QtCore.Signal()
 
-    collecting = QtCore.pyqtSignal()
-    validating = QtCore.pyqtSignal()
-    extracting = QtCore.pyqtSignal()
-    integrating = QtCore.pyqtSignal()
+    collecting = QtCore.Signal()
+    validating = QtCore.Signal()
+    extracting = QtCore.Signal()
+    integrating = QtCore.Signal()
 
-    repairing = QtCore.pyqtSignal()
-    stopping = QtCore.pyqtSignal()
-    saving = QtCore.pyqtSignal()
-    initialising = QtCore.pyqtSignal()
-    acting = QtCore.pyqtSignal()
-    acted = QtCore.pyqtSignal()
+    repairing = QtCore.Signal()
+    stopping = QtCore.Signal()
+    saving = QtCore.Signal()
+    initialising = QtCore.Signal()
+    acting = QtCore.Signal()
+    acted = QtCore.Signal()
 
     # A plug-in/instance pair is about to be processed
-    about_to_process = QtCore.pyqtSignal(object, object)
+    about_to_process = QtCore.Signal(object, object)
 
-    changed = QtCore.pyqtSignal()
+    changed = QtCore.Signal()
 
-    ready = QtCore.pyqtSignal()
-    saved = QtCore.pyqtSignal()
-    finished = QtCore.pyqtSignal()
-    initialised = QtCore.pyqtSignal()
-    commented = QtCore.pyqtSignal()
-    commenting = QtCore.pyqtSignal(str, arguments=["comment"])
+    ready = QtCore.Signal()
+    saved = QtCore.Signal()
+    finished = QtCore.Signal()
+    initialised = QtCore.Signal()
+    commented = QtCore.Signal()
+    commenting = QtCore.Signal(str, arguments=["comment"])
 
-    state_changed = QtCore.pyqtSignal(str, arguments=["state"])
+    state_changed = QtCore.Signal(str, arguments=["state"])
 
     # Statically expose these members to the QML run-time.
     itemModel = qtproperty(lambda self: self.data["models"]["item"])
@@ -61,13 +61,13 @@ class Controller(QtCore.QObject):
     resultModel = qtproperty(lambda self: self.data["models"]["result"])
     resultProxy = qtproperty(lambda self: self.data["proxies"]["result"])
 
-    def __init__(self, host, parent=None, targets=[]):
+    def __init__(self, host, parent=None, targets=None):
         super(Controller, self).__init__(parent)
 
         # Connection to host
         self.host = host
 
-        self.targets = targets
+        self.targets = targets or []
 
         self.data = {
             "models": {
@@ -75,7 +75,7 @@ class Controller(QtCore.QObject):
                 "result": models.ResultModel(),
             },
             "comment": "",
-            "firstRun": True
+            "firstRun": True,
         }
 
         self.data.update({
@@ -107,8 +107,11 @@ class Controller(QtCore.QObject):
             },
             "state": {
                 "is_running": False,
+                "readyCount": 0,
                 "current": None,
-                "all": list()
+                "all": list(),
+
+                "testPassed": False,
             }
         })
 
@@ -123,6 +126,7 @@ class Controller(QtCore.QObject):
         self.info.connect(self.on_info)
         self.error.connect(self.on_error)
         self.finished.connect(self.on_finished)
+        self.ready.connect(self.on_ready)
         self.show.connect(self.on_show)
 
         # NOTE: Listeners to this signal are run in the main thread
@@ -289,20 +293,20 @@ class Controller(QtCore.QObject):
         machine.start()
         return machine
 
-    @QtCore.pyqtSlot(result=str)
+    @QtCore.Slot(result=str)
     def comment(self):
         """Return first line of comment"""
         return self.data["comment"]
 
-    @QtCore.pyqtProperty(str, notify=state_changed)
+    @QtCore.Property(str, notify=state_changed)
     def state(self):
         return self.data["state"]["current"]
 
-    @QtCore.pyqtProperty(bool, notify=commented)
+    @QtCore.Property(bool, notify=commented)
     def hasComment(self):
         return True if self.data["comment"] else False
 
-    @QtCore.pyqtProperty(bool, constant=True)
+    @QtCore.Property(bool, constant=True)
     def commentEnabled(self):
         return "comment" in self.host.cached_context.data
 
@@ -310,7 +314,7 @@ class Controller(QtCore.QObject):
     def states(self):
         return self.data["state"]["all"]
 
-    @QtCore.pyqtSlot(result=float)
+    @QtCore.Slot(result=float)
     def time(self):
         return time.time()
 
@@ -325,7 +329,6 @@ class Controller(QtCore.QObject):
 
         """
 
-        test = pyblish.logic.registered_test()
         state = {
             "nextOrder": None,
             "ordersWithError": set()
@@ -349,10 +352,14 @@ class Controller(QtCore.QObject):
                     signals.pop(order).emit()
 
             if not self.data["state"]["is_running"]:
-                raise StopIteration("Stopped")
+                return StopIteration("Stopped")
 
-            if test(**state):
-                raise StopIteration("Stopped due to %s" % test(**state))
+            test_result = self.host.test(**state)
+            if test_result:
+                self.data["state"]["testPassed"] = False
+                return StopIteration("Stopped due to %s" % test_result)
+
+            self.data["state"]["testPassed"] = True
 
             try:
                 # Notify GUI before commencing remote processing
@@ -361,7 +368,7 @@ class Controller(QtCore.QObject):
                 result = self.host.process(plug, context, instance)
 
             except Exception as e:
-                raise StopIteration("Unknown error: %s" % e)
+                return StopIteration("Unknown error: %s" % e)
 
             else:
                 # Make note of the order at which the
@@ -372,7 +379,7 @@ class Controller(QtCore.QObject):
 
             yield result
 
-    @QtCore.pyqtSlot(int, result=QtCore.QVariant)
+    @QtCore.Slot(int, result="QVariant")
     def getPluginActions(self, index):
         """Return actions from plug-in at `index`
 
@@ -434,7 +441,7 @@ class Controller(QtCore.QObject):
 
         return remaining_actions
 
-    @QtCore.pyqtSlot(str)
+    @QtCore.Slot(str)
     def runPluginAction(self, action):
         if "acting" in self.states:
             return self.error.emit("Busy")
@@ -486,7 +493,7 @@ class Controller(QtCore.QObject):
 
         util.defer(run, callback=on_finished)
 
-    @QtCore.pyqtSlot(int)
+    @QtCore.Slot(int)
     def toggleInstance(self, index):
         models = self.data["models"]
         proxies = self.data["proxies"]
@@ -501,7 +508,7 @@ class Controller(QtCore.QObject):
         else:
             self.error.emit("Cannot toggle")
 
-    @QtCore.pyqtSlot(bool, str)
+    @QtCore.Slot(bool, str)
     def toggleSection(self, checkState, sectionLabel):
         model = self.data["models"]["item"]
 
@@ -530,7 +537,7 @@ class Controller(QtCore.QObject):
                 if item.isToggled != checkState and item.optional:
                     self.__toggle_item(model, model.items.index(item))
 
-    @QtCore.pyqtSlot(bool, str)
+    @QtCore.Slot(bool, str)
     def hideSection(self, hideState, sectionLabel):
         model = self.data["models"]["item"]
 
@@ -544,7 +551,7 @@ class Controller(QtCore.QObject):
             if item.itemType == "section" and item.name == sectionLabel:
                 self.__hide_item(model, model.items.index(item), hideState)
 
-    @QtCore.pyqtSlot(int, result=QtCore.QVariant)
+    @QtCore.Slot(int, result="QVariant")
     def pluginData(self, index):
         models = self.data["models"]
         proxies = self.data["proxies"]
@@ -554,7 +561,7 @@ class Controller(QtCore.QObject):
         source_index = source_qindex.row()
         return self.__item_data(models["item"], source_index)
 
-    @QtCore.pyqtSlot(int, result=QtCore.QVariant)
+    @QtCore.Slot(int, result="QVariant")
     def instanceData(self, index):
         models = self.data["models"]
         proxies = self.data["proxies"]
@@ -564,7 +571,7 @@ class Controller(QtCore.QObject):
         source_index = source_qindex.row()
         return self.__item_data(models["item"], source_index)
 
-    @QtCore.pyqtSlot(int)
+    @QtCore.Slot(int)
     def togglePlugin(self, index):
         models = self.data["models"]
         proxies = self.data["proxies"]
@@ -579,7 +586,7 @@ class Controller(QtCore.QObject):
         else:
             self.error.emit("Cannot toggle")
 
-    @QtCore.pyqtSlot(str, str, str, str)
+    @QtCore.Slot(str, str, str, str)
     def exclude(self, target, operation, role, value):
         """Exclude a `role` of `value` at `target`
 
@@ -604,7 +611,7 @@ class Controller(QtCore.QObject):
         else:
             raise TypeError("operation must be either `add` or `remove`")
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def save(self):
         # Deprecated
         return
@@ -659,6 +666,11 @@ class Controller(QtCore.QObject):
         self.host.update(key="comment", value=comment)
         self.host.emit("commented", comment=comment)
 
+    def is_ready(self):
+        count = self.data["state"]["readyCount"]
+        util.wait(self.ready, 1000)
+        return self.data["state"]["readyCount"] == count + 1
+
     # Event handlers
 
     def on_commenting(self, comment):
@@ -707,6 +719,9 @@ class Controller(QtCore.QObject):
             s.name for s in self.machine.configuration()
         )
 
+    def on_ready(self):
+        self.data["state"]["readyCount"] += 1
+
     def on_finished(self):
         self.data["models"]["item"].reset_status()
 
@@ -728,12 +743,12 @@ class Controller(QtCore.QObject):
 
     # Slots
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def stop(self):
         self.data["state"]["is_running"] = False
         self.stopping.emit()
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def reset(self):
         """Request that host re-discovers plug-ins and re-processes selectors
 
@@ -880,7 +895,7 @@ class Controller(QtCore.QObject):
 
         util.defer(self.host.reset, callback=on_reset)
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def publish(self):
         """Start asynchonous publishing
 
@@ -922,9 +937,22 @@ class Controller(QtCore.QObject):
         def on_finished():
             self.host.emit("published", context=None)
 
+            if not self.data["state"]["testPassed"]:
+                # Possible stopped on validation fail or the extraction has
+                # been interrupted. Depend on the continual processing test.
+                return
+
+            # If there are instance that has error after the test passed,
+            # prompt publish failed message to footer.
+            model = self.data["models"]["item"]
+            for instance in models.ItemIterator(model.instances):
+                if instance.hasError:
+                    self.info.emit("Published, with errors..")
+                    break
+
         util.defer(get_data, callback=on_data_received)
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def validate(self):
         """Start asynchonous validation
 
@@ -966,7 +994,7 @@ class Controller(QtCore.QObject):
 
         util.defer(get_data, callback=on_data_received)
 
-    def run(self, plugins, context, callback=None, callback_args=[]):
+    def run(self, plugins, context, callback=None, callback_args=None):
         """Commence asynchronous tasks
 
         This method runs through the provided `plugins` in
@@ -983,6 +1011,7 @@ class Controller(QtCore.QObject):
             callback_args (list, optional): Arguments passed to callback
 
         """
+        callback_args = callback_args or []
 
         # if "ready" not in self.states:
         #     return self.error.emit("Not ready")
@@ -1034,6 +1063,7 @@ class Controller(QtCore.QObject):
             item.isToggled = data.get("publish", True)
             item.optional = data.get("optional", True)
             item.category = data.get("category", data["family"])
+            item.label = data.get("label", None)
 
             families = [data["family"]]
             families.extend(data.get("families", []))
@@ -1083,7 +1113,7 @@ class Controller(QtCore.QObject):
         iterator = self.iterator(plugins, context)
         util.defer(lambda: next(iterator), callback=on_next)
 
-    @QtCore.pyqtSlot(int)
+    @QtCore.Slot(int)
     def repairPlugin(self, index):
         """
 
@@ -1177,7 +1207,7 @@ def iterator(plugins, context):
 
         message = test(**state)
         if message:
-            raise StopIteration("Stopped due to %s" % message)
+            return StopIteration("Stopped due to %s" % message)
 
         instances = pyblish.api.instances_by_plugin(context, plugin)
         if plugin.__instanceEnabled__:
