@@ -2,10 +2,9 @@ import re
 import time
 import logging
 
-from PyQt5 import QtCore
-
 from . import util, settings
 from .vendor import six
+from .vendor.Qt5 import QtCore
 
 
 defaults = {
@@ -109,7 +108,7 @@ class PropertyType(type(QtCore.QObject)):
     def __new__(cls, name, bases, attrs):
         """Convert class properties into pyqtProperties
 
-        For use in conjuction with the :func:Item factory function.
+        For use in conjunction with the :func:Item factory function.
 
         """
 
@@ -117,7 +116,7 @@ class PropertyType(type(QtCore.QObject)):
             if key.startswith("__"):
                 continue
 
-            notify = QtCore.pyqtSignal()
+            notify = QtCore.Signal()
 
             def set_data(key, value):
                 def set_data(self, value):
@@ -127,8 +126,8 @@ class PropertyType(type(QtCore.QObject)):
                 return set_data
 
             attrs[key + "Changed"] = notify
-            attrs[key] = QtCore.pyqtProperty(
-                type(value) if value is not None else QtCore.QVariant,
+            attrs[key] = QtCore.Property(
+                type(value) if value is not None else "QVariant",
                 fget=lambda self, k=key: getattr(self, cls.prefix + k, None),
                 fset=set_data(key, value),
                 notify=notify)
@@ -145,7 +144,7 @@ class AbstractItem(QtCore.QObject):
 
     """
 
-    __datachanged__ = QtCore.pyqtSignal(QtCore.QObject)
+    __datachanged__ = QtCore.Signal(QtCore.QObject)
 
     def __str__(self):
         return self.name
@@ -199,7 +198,7 @@ class AbstractModel(QtCore.QAbstractListModel):
         super(AbstractModel, self).__init__(parent)
         self.items = util.ItemList(key="id")
 
-    @QtCore.pyqtSlot(int, result=QtCore.QObject)
+    @QtCore.Slot(int, result=QtCore.QObject)
     def item(self, index):
         return self.items[index]
 
@@ -247,7 +246,7 @@ class AbstractModel(QtCore.QAbstractListModel):
             except Exception:
                 pass
 
-        return QtCore.QVariant()
+        return "QVariant"
 
     def roleNames(self):
         return {
@@ -309,7 +308,7 @@ class ItemModel(AbstractModel):
 
         self.endResetModel()
 
-    @QtCore.pyqtSlot(QtCore.QVariant)
+    @QtCore.Slot("QVariant")
     def add_plugin(self, plugin):
         """Append `plugin` to model
 
@@ -369,6 +368,10 @@ class ItemModel(AbstractModel):
         }.get(item["type"], "Other")
 
         for action in item["actions"]:
+            if action["__type__"] != "action":
+                # Consider only Actions, ignore Categories
+                continue
+
             if action["on"] == "all":
                 item["actionsIconVisible"] = True
 
@@ -377,7 +380,7 @@ class ItemModel(AbstractModel):
         item = self.add_item(item)
         self.plugins.append(item)
 
-    @QtCore.pyqtSlot(QtCore.QVariant)
+    @QtCore.Slot("QVariant")
     def add_instance(self, instance):
         """Append `instance` to model
 
@@ -441,7 +444,7 @@ class ItemModel(AbstractModel):
 
         return item
 
-    @QtCore.pyqtSlot(QtCore.QVariant)
+    @QtCore.Slot("QVariant")
     def add_context(self, context, label=None):
         """Append `context` to model
 
@@ -532,7 +535,8 @@ class ItemModel(AbstractModel):
                     if action["on"] == "processed" and not item.processed:
                         actions.remove(action)
 
-                if actions:
+                if any(action["__type__"] == "action" for action in actions):
+                    # Consider only Actions, ignore Categories
                     item.actionsIconVisible = True
 
             # Update section item
@@ -599,7 +603,7 @@ class ItemModel(AbstractModel):
 
 class ResultModel(AbstractModel):
 
-    added = QtCore.pyqtSignal()
+    added = QtCore.Signal()
 
     def add_item(self, item):
         item_ = defaults["result"].copy()
@@ -739,21 +743,21 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
 
         self.setFilterCaseSensitivity(QtCore.Qt.CaseInsensitive)
 
-    @QtCore.pyqtSlot(int, result=QtCore.QObject)
+    @QtCore.Slot(int, result=QtCore.QObject)
     def item(self, index):
         index = self.index(index, 0, QtCore.QModelIndex())
         index = self.mapToSource(index)
         model = self.sourceModel()
         return model.items[index.row()]
 
-    @QtCore.pyqtSlot(str, result=QtCore.QObject)
+    @QtCore.Slot(str, result=QtCore.QObject)
     def itemByName(self, name):
         model = self.sourceModel()
         for item in model.items:
             if name == item.name:
                 return item
 
-    @QtCore.pyqtSlot(str, str)
+    @QtCore.Slot(str, str)
     def add_exclusion(self, role, value):
         """Exclude item if `role` equals `value`
 
@@ -765,7 +769,7 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
 
         self._add_rule(self.excludes, role, value)
 
-    @QtCore.pyqtSlot(str, str)
+    @QtCore.Slot(str, str)
     def remove_exclusion(self, role, value=None):
         """Remove exclusion rule
 
@@ -790,11 +794,11 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
 
         self._set_rules(self.excludes, rules)
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def clear_exclusion(self):
         self._clear_group(self.excludes)
 
-    @QtCore.pyqtSlot(str, str)
+    @QtCore.Slot(str, str)
     def add_inclusion(self, role, value):
         """Include item if `role` equals `value`
 
@@ -806,7 +810,7 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
 
         self._add_rule(self.includes, role, value)
 
-    @QtCore.pyqtSlot(str, str)
+    @QtCore.Slot(str, str)
     def remove_inclusion(self, role, value=None):
         """Remove exclusion rule"""
         self._remove_rule(self.includes, role, value)
@@ -814,7 +818,7 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
     def set_inclusion(self, rules):
         self._set_rules(self.includes, rules)
 
-    @QtCore.pyqtSlot()
+    @QtCore.Slot()
     def clear_inclusion(self):
         self._clear_group(self.includes)
 
@@ -880,6 +884,6 @@ class ProxyModel(QtCore.QSortFilterProxyModel):
         return super(ProxyModel, self).filterAcceptsRow(
             source_row, source_parent)
 
-    @QtCore.pyqtSlot(result=int)
+    @QtCore.Slot(result=int)
     def rowCount(self, parent=QtCore.QModelIndex()):
         return super(ProxyModel, self).rowCount(parent)
